# -*- coding: utf-8 -*-
#
# File: AppInstall.py
#
# Copyright (c) 2007 by []
#
# GNU General Public License (GPL)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#
#
# Install Replaces an existing metadata tool if one exists
# Uninstall removes the metadata tool and installs the plone default tool

from Products.AnnotatableMetadata.config import PROJECTNAME
from Products.AnnotatableMetadata.AnnotatableMetadata  import AnnotatableMetadata
from Products.CMFPlone.MetadataTool import MetadataTool
from Products.CMFCore.utils import getToolByName
from Products.AnnotatableMetadata.config import TOOL_ID, METADATASET_ID
from StringIO import StringIO

# the ID of the portal_metadata tool
toolId = 'portal_metadata'

def install(self):
    """ External Method to install AnnotatableMetadata's portal_metadata tool """
    out = StringIO()
    print >> out, "Custom Installation log of %s:" % PROJECTNAME
    mds_portal_type = 'metadataset'
    portal = self.portal_url.getPortalObject()
    amtool = getattr(portal, TOOL_ID)
    amtool.invokeFactory(mds_portal_type, METADATASET_ID)

    # the atse tool registers schemata of objects by creating a test instance of the object on itself.
    # An object which is not globally addable can therefore not be used by the atse. So as a quick fix,
    # we will allow our object type metadataset to be added to the atse tool.
    pti = getToolByName(portal, 'portal_types')
    atse = pti.getTypeInfo('ATSchemaEditorTool')
    ACT = atse.allowed_content_types
    if mds_portal_type not in ACT:
        ACT += (mds_portal_type,)
        atse.allowed_content_types = ACT
    amtool.atse_registerPortalType(mds_portal_type)

    # remove unneccessary standard keys from the schema of the metadataset
    S = amtool.atse_getSchemaById(mds_portal_type)
    S.delSchemata('metadata')
    amtool._schemas[mds_portal_type] = S
    amtool._p_changed = 1

    amtool.atse_delField(mds_portal_type, 'default', 'id')
    # Schema cannot be empty so we add a default element
    amtool.atse_addField(mds_portal_type, 'default', '', 'default_metadata')
    # and then kill the last one
    amtool.atse_delField(mds_portal_type, 'default', 'title')
    # updating all portal_types
    uid = portal.portal_url.getPortalPath() + '/schema_editor_tool/atse-ptypedummy-metadataset'
    getToolByName(portal, 'portal_catalog').uncatalog_object(uid)
    amtool.atse_updateManagedSchema(mds_portal_type, '', update_all=True)


    # there is a dummy object called atse-ptypedummy-portaltype which breaks the updating of the schema in atse

    print >>out,"Configured annotatable portal_metadata"

    return out.getvalue()

def uninstall(self):
    """ External Method to uninstall AnnotatableMetadata's portal_metadata tool and reinstall the default """
    out = StringIO()
    print >> out, "Custom Uninstallation log of %s:" % PROJECTNAME
    portal = self.portal_url.getPortalObject()

    pti = getToolByName(portal, 'portal_types')
    atse = pti.getTypeInfo('ATSchemaEditorTool')
    ACT = atse.allowed_content_types
    if 'metadataset' in ACT:
        NACT = list(ACT)
        NACT.remove('metadataset')
        atse.allowed_content_types = tuple(NACT)

    print >>out, "Custom Uninstallation done"

    return out.getvalue()
